{-|
Module      : Data.Graph.Clustering.ILouvain
Description : Purely functional (Inductive) Louvain clustering
Copyright   : (c) Alexandre Delanoë, CNRS, 2020-Present
License     : AGPL + CECILL v3
Maintainer  : alexandre.delanoe+louvain@iscpif.fr
Stability   : experimental
Portability : POSIX

ILouvain: really inductive Graph

-}

{-# LANGUAGE ConstrainedClassMethods #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE InstanceSigs      #-}
{-# LANGUAGE NoImplicitPrelude #-}

module Data.Graph.Clustering.ILouvain
  where

import Data.List (zip, cycle)
import Protolude hiding (empty)
import Data.Graph.Inductive


------------------------------------------------------------------------
-- Recursive Graph 
data RGraph = Empty | Gr RGraph Double
  deriving (Show, Eq)
------------------------------------------------------------------------
-- Spoon Graph
--    1
--   / \
--  2   3
--   \ /
--    4
--    |
--    5

ns :: [LNode RGraph]
ns = zip [1..6] (cycle [Empty])

es :: [LEdge Double]
es = [ (1, 2, 1.0)
     , (1, 3, 1.0)
     , (2, 4, 1.0)
     , (3, 4, 1.0)
     , (4, 5, 1.0)
     ]

spoon :: Gr RGraph Double
spoon = mkGraph ns es

------------------------------------------------------------------------
-- Path in the Graph
path :: Graph gr => gr a b -> [[Node]]
path g = map (\xs -> dfs xs g ) (components g)

------------------------------------------------------------------------

